package nl.iobyte.themepark;

import nl.iobyte.menuapi.MenuAPI;
import nl.iobyte.themepark.command.MainCommand;
import nl.iobyte.themepark.command.MenuCommand;
import nl.iobyte.themepark.command.RidecountCommand;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.database.DB;
import nl.iobyte.themepark.database.DBManager;
import nl.iobyte.themepark.database.MySQL;
import nl.iobyte.themepark.database.SQLite;
import nl.iobyte.themepark.listener.AttractionListener;
import nl.iobyte.themepark.listener.PlayerListener;
import nl.iobyte.themepark.listener.RegionListener;
import nl.iobyte.themepark.listener.SignListener;
import nl.iobyte.themepark.menu.AttractionLoader;
import nl.iobyte.themepark.sign.SignManager;
import nl.iobyte.workchain.thread.WorkThreader;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandMap;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

public class ThemeParkPlugin extends JavaPlugin {

    private static ThemeParkPlugin instance;
    private YamlConfig settings, messages, data, regions, signs;
    private WorkThreader threader;

    public void onEnable() {
        instance = this;
        loadConfigs();
        loadDatabases();
        loadWorkers();
        loadListeners();
        loadCommands();
        setupTrainCarts();
        AttractionLoader.load();
    }

    private void loadConfigs() {
        settings = new YamlConfig("settings", this);
        messages = new YamlConfig("messages", this);
        data = new YamlConfig("data", this);
        regions = new YamlConfig("regions", this);
        signs = new YamlConfig("signs", this);
    }

    private void loadDatabases() {
        DBManager.addDatabase("data", new SQLite("data"));

        FileConfiguration fc = settings.getConfig();
        if(fc.getBoolean("mysql.enabled")) {
            String url = fc.getString("mysql.url");
            String host = fc.getString("mysql.host");
            int port = fc.getInt("mysql.port");
            String database = fc.getString("mysql.database");
            String username = fc.getString("mysql.user");
            String password = fc.getString("mysql.pass");
            DBManager.addDatabase("public", new MySQL(url, host, port, database, username, password));
        }

        loadTables();
    }

    private void loadTables() {
        DB db = DBManager.getDatabase("data");
        DBManager.createTable(db, "CREATE TABLE IF NOT EXISTS counts(uuid VARCHAR(255) NOT NULL, attraction_id VARCHAR(255) NOT NULL, points BIGINT(20), created_at TEXT DEFAULT CURRENT_DATE, UNIQUE(uuid, attraction_id, created_at))");

        db = DBManager.getDatabase("public");
        if(db == null)
            return;

        DBManager.createTable(db, "CREATE TABLE IF NOT EXISTS attraction (id VARCHAR(256) NOT NULL, name VARCHAR(256) NOT NULL, region_id VARCHAR(256) NOT NULL, type VARCHAR(256) NOT NULL, status VARCHAR(256) NOT NULL, PRIMARY KEY(id))");
        DBManager.createTable(db, "CREATE TABLE IF NOT EXISTS region (id VARCHAR(256) NOT NULL, name VARCHAR(256) NOT NULL, PRIMARY KEY(id))");
        DBManager.createTable(db, "CREATE TABLE IF NOT EXISTS status (statusId VARCHAR(256) NOT NULL, statusName VARCHAR(256) NOT NULL, PRIMARY KEY(statusId))");
    }

    private void loadWorkers() {
        int workers = settings.getConfig().getInt("workers");
        if(workers < 1)
            workers = 1;

        threader = new WorkThreader(workers);
        threader.createWorkers();
        threader.start();
    }

    private void loadListeners() {
        PluginManager pm = Bukkit.getPluginManager();
        pm.registerEvents(new PlayerListener(), this);
        pm.registerEvents(new AttractionListener(), this);
        pm.registerEvents(new RegionListener(), this);
        pm.registerEvents(new SignListener(), this);
        MenuAPI.register(this);
    }

    private void loadCommands() {
        getCommand("status").setExecutor(new MenuCommand());
        getCommand("ridecount").setExecutor(new RidecountCommand());
        String command = settings.getConfig().getString("command");
        if(command == null || command.isEmpty())
            return;

        try {
            final Field bukkitCommandMap = Bukkit.getServer().getClass().getDeclaredField("commandMap");

            bukkitCommandMap.setAccessible(true);
            CommandMap commandMap = (CommandMap) bukkitCommandMap.get(Bukkit.getServer());

            commandMap.register(command, new MainCommand(command));
        } catch(ReflectiveOperationException e) {
            e.printStackTrace();
        }
    }

    private void setupTrainCarts() {
        try {
            Plugin plugin = Bukkit.getPluginManager().getPlugin("Train_Carts");
            if (plugin != null && plugin.isEnabled()) {
                Class<?> clazz = Class.forName("com.bergerkiller.bukkit.tc.signactions.SignAction");
                Class<?> sign = Class.forName("nl.iobyte.themepark.traincarts.TCRideCount");
                Method method = clazz.getMethod("register", clazz);
                method.invoke(clazz, sign.newInstance());
            }
        } catch (Exception e) {
            System.out.println("["+getName()+"] TrainCarts not found!");
        }
    }

    public void onDisable() {
        threader.stop();
        DBManager.shutdown();
        SignManager.saveSigns();
        instance = null;
    }

    public static ThemeParkPlugin getInstance() {
        return instance;
    }

    public YamlConfig getSettings() {
        return settings;
    }

    public YamlConfig getMessages() {
        return messages;
    }

    public YamlConfig getData() {
        return data;
    }

    public YamlConfig getRegions() {
        return regions;
    }

    public YamlConfig getSigns() {
        return signs;
    }

}
