package nl.iobyte.themepark.sign;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.utils.LocationSerializer;
import nl.iobyte.themepark.config.YamlConfig;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Sign;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SignManager {

    private static YamlConfig config = ThemeParkPlugin.getInstance().getSigns();
    private static HashMap<Attraction, ArrayList<StatusSign>> signs = new HashMap<>();

    public static void loadSigns(Attraction attraction) {
        if(attraction == null) {
            System.out.println("SignManager: No attraction passed");
            return;
        }

        if(!config.getConfig().contains("signs." + attraction.getId())) {
            System.out.println("SignManager: No signs for attraction with ID: "+attraction.getId());
            return;
        }

        if(!config.getConfig().isList("signs." + attraction.getId())) {
            System.out.println("SignManager: No signs for attraction with ID: "+attraction.getId());
            config.getConfig().set("signs." + attraction.getId(), null);
            config.save();
            return;
        }

        ArrayList<StatusSign> array = new ArrayList<>();
        List<String> locations = config.getConfig().getStringList("signs." + attraction.getId());
        for(String string : locations) {
            Location location = LocationSerializer.toLocation(string);
            if(location == null) {
                System.out.println("SignManager: Invalid location for sign: "+string);
                continue;
            }


            if(!(location.getBlock().getState() instanceof Sign)) {
                System.out.println("SignManager: Block is not a sign at: "+string);
                continue;
            }

            array.add(new StatusSign(attraction, location));
        }

        signs.put(attraction, array);
        System.out.println("SignManager: Loaded "+array.size()+" signs for attraction: "+attraction.getId());
    }

    public static void saveSigns() {
        for(Map.Entry<Attraction, ArrayList<StatusSign>> entry : signs.entrySet()) {
            List<String> array = new ArrayList<>();
            for(StatusSign sign : entry.getValue())
                if(LocationSerializer.toString(sign.getLocation()) != null)
                    array.add(LocationSerializer.toString(sign.getLocation()));

            config.getConfig().set("signs." + entry.getKey().getId(), array);
        }

        config.save();
    }

    public static void addSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null)
            return;

        if(!(sign.getLocation().getBlock().getState() instanceof Sign))
            return;

        if(signs.containsKey(sign.getAttraction())) {
            signs.get(sign.getAttraction()).add(sign);
            return;
        }

        ArrayList<StatusSign> array = new ArrayList<>();
        array.add(sign);
        signs.put(sign.getAttraction(), array);
    }

    public static boolean hasSigns(Attraction attraction) {
        if(attraction == null)
            return false;

        return signs.containsKey(attraction) && (signs.get(attraction).size() > 0);
    }

    public static boolean isSign(Attraction attraction, Location location) {
        if(attraction == null || location == null)
            return false;

        ArrayList<StatusSign> array = signs.get(attraction);
        if(array == null)
            return false;

        String loc = LocationSerializer.toString(location);
        if(loc == null)
            return false;

        for(StatusSign sign : array) {
            String str = LocationSerializer.toString(sign.getLocation());
            if(str == null)
                continue;

            if(loc.equals(str))
                return true;
        }

        return false;
    }

    public static void update(Attraction attraction) {
        if(attraction == null)
            return;

        if(!hasSigns(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.update();
    }

    public static void remove(Attraction attraction) {
        if(attraction == null || !signs.containsKey(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.getLocation().getBlock().setType(Material.AIR);

        signs.remove(attraction);
        config.getConfig().set("signs." + attraction.getId(), null);
        config.save();
    }

    public static void removeSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null || sign.getAttraction() == null)
            return;

        if(!hasSigns(sign.getAttraction()))
            return;

        Location loc1 = sign.getLocation();
        String str1 = LocationSerializer.toString(loc1);
        ArrayList<StatusSign> array = signs.get(sign.getAttraction());
        for(StatusSign s : new ArrayList<>(array)) {
            Location loc2 = s.getLocation();

            if(loc1 == null || loc2 == null)
                return;

            String str2 = LocationSerializer.toString(loc2);
            if(str1 == null || str2 == null)
                return;

            if(!str1.equals(str2))
                return;

            array.remove(s);
        }

        if(array.isEmpty()) {
            signs.remove(sign.getAttraction());
            config.getConfig().set("signs." + sign.getAttraction().getId(), null);
            config.save();
            return;
        }

        signs.put(sign.getAttraction(), array);
    }

}